@@ -173,6 +173,11 @@ function configure_cloudkitty {
         iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} port ${CLOUDKITTY_INFLUXDB_PORT}
     fi
 
+    if [ "$CLOUDKITTY_STORAGE_BACKEND" == "elasticsearch" ]; then
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} host ${CLOUDKITTY_ELASTICSEARCH_HOST}
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} index_name ${CLOUDKITTY_ELASTICSEARCH_INDEX}
+    fi
+
     # collect
     iniset $CLOUDKITTY_CONF collect collector $CLOUDKITTY_COLLECTOR
     iniset $CLOUDKITTY_CONF "collector_${CLOUDKITTY_COLLECTOR}" auth_section authinfos
@@ -195,7 +200,7 @@ function configure_cloudkitty {
     iniset $CLOUDKITTY_CONF database connection $dburl
 
     # keystone middleware
-    configure_auth_token_middleware $CLOUDKITTY_CONF cloudkitty $CLOUDKITTY_AUTH_CACHE_DIR
+    configure_keystone_authtoken_middleware $CLOUDKITTY_CONF cloudkitty
 
     if is_service_enabled ck-api && [ "$CLOUDKITTY_USE_MOD_WSGI" == "True" ]; then
         _cloudkitty_config_apache_wsgi
@@ -237,6 +242,12 @@ function create_influxdb_database {
     fi
 }
 
+function create_elasticsearch_index {
+    if [ "$CLOUDKITTY_STORAGE_BACKEND" == "elasticsearch" ]; then
+        curl -XPUT "${CLOUDKITTY_ELASTICSEARCH_HOST}/${CLOUDKITTY_ELASTICSEARCH_INDEX}"
+    fi
+}
+
 # init_cloudkitty() - Initialize CloudKitty database
 function init_cloudkitty {
     # Delete existing cache
@@ -253,6 +264,7 @@ function init_cloudkitty {
     recreate_database cloudkitty utf8
 
     create_influxdb_database
+    create_elasticsearch_index
 
     # Migrate cloudkitty database
     $CLOUDKITTY_BIN_DIR/cloudkitty-dbsync upgrade
@@ -289,6 +301,22 @@ function install_influx {
     sudo systemctl start influxdb || sudo systemctl restart influxdb
 }
 
+function install_opensearch {
+    OPENSEARCH_HOME=/usr/share/opensearch
+    local opensearch_file=$(get_extra_file "https://artifacts.opensearch.org/releases/bundle/opensearch/1.3.6/opensearch-1.3.6-linux-x64.tar.gz")
+    sudo mkdir -p $OPENSEARCH_HOME
+    sudo tar -xzpf ${opensearch_file} -C $OPENSEARCH_HOME --strip-components=1
+    sudo mkdir -p $OPENSEARCH_HOME/data /var/log/opensearch
+    sudo chown -R $STACK_USER $OPENSEARCH_HOME /var/log/opensearch
+    cat - <<EOF | sudo tee $OPENSEARCH_HOME/config/opensearch.yml >/dev/null
+discovery.type: single-node
+path.data: /usr/share/opensearch/data
+path.logs: /var/log/opensearch
+plugins.security.disabled: true
+EOF
+    _run_under_systemd opensearch "$OPENSEARCH_HOME/bin/opensearch"
+}
+
 # install_cloudkitty() - Collect source and prepare
 function install_cloudkitty {
     git_clone $CLOUDKITTY_REPO $CLOUDKITTY_DIR $CLOUDKITTY_BRANCH
@@ -296,6 +324,8 @@ function install_cloudkitty {
 
     if [ $CLOUDKITTY_STORAGE_BACKEND == 'influxdb' ]; then
         install_influx
+    elif [ $CLOUDKITTY_STORAGE_BACKEND == 'elasticsearch' ]; then
+        install_opensearch
     fi
 }
 
